
class vector(object):
	""" Simple point or 2d vector class """

	def __init__(self, x, y):
		self.x = x
		self.y = y

	def __repr__(self):
		return 'vector({x}, {y})'.format(x=self.x, y=self.y)

	#def __str__(self):
		#return '({x},{y})'.format(x=self.x, y=self.y)

	def __add__(self, rhs):
		return vector(self.x + rhs.x, self.y + rhs.y)

	def __sub__(self, rhs):
		return vector(self.x - rhs.x, self.y - rhs.y)

	def __eq__(self, rhs):
		return self.x == rhs.x and self.y == rhs.y

	def __nonzero__(self):
		return self.x != 0 or self.y != 0

	def scale(self, f):
		return vector(self.x * f, self.y * f)

	def count(self):
		return 1

	def move_relative(self, dx, dy):
		self.x += dx
		self.y += dy

	def to_tuple(self):
		return (self.x, self.y)

	@staticmethod
	def dot_product(a, b):
		return a.x*b.x + a.y*b.y

# constants
vector.Up    = vector( 0, 1)
vector.Down  = vector( 0,-1)
vector.Left  = vector(-1, 0)
vector.Right = vector( 1, 0)



class rectangle(object):
	""" Simple 2d rectangle class """

	def __init__(self, x, y, w, h):
		self.point = vector(x,y)
		self.size  = vector(w,h)

	def __repr__(self):
		(x,y,w,h) = self.raw()
		return 'rectangle({x},{y},{w},{h})'.format(x=x,y=y,w=w,h=h)

	# returns (x,y,w,h)
	def raw(self):
		return (self.point.x, self.point.y, self.size.x, self.size.y)

	def move(self, dx, dy):
		self.point += vector(dx,dy)


	# returns a tuple for GL_QUADS
	def to_tuple(self):
		(x,y,w,h) = self.raw()
		return (x,y, x+w,y, x+w,y+h, x,y+h)

	def to_line_tuple(self):
		(x,y,w,h) = self.raw()
		return (x,y, x+w,y, x+w,y+h, x,y+h)

	def top(self):
		return (self.point.y + self.size.y)

	def contains(self, rect):
		for p in rect.points():
			if not self.contains_p(p):
				return False
		return True

	def contains_p(self, p):
		(x,y,w,h) = self.raw()
		return p.x >= x and p.x <= x+w and p.y >= y and p.y <= y+h

	def merge(self, p):
		(x,y,w,h) = self.raw()

		if(p.x < x):
			x = p.x
			w += x-p.x
		if(p.y < y):
			y = p.y
			h += x-p.x
		if(p.x > x+w):
			w = p.x - x
		if(p.y > y+h):
			h = p.y - y

		return rectangle(x,y,w,h)
	
	def bottom_left(self):
		return self.point

	def top_left(self):
		return vector(self.point.x, self.point.y + self.size.y)

	def bottom_right(self):
		return vector(self.point.x + self.size.x, self.point.y)

	def top_right(self):
		return self.point + self.size

	# separate coordinate versions
	def contains_xy(self, x,y):
		return self.contains_p(vector(x,y))

	def merge_xy(self, x,y):
		return self.merge(vector(x,y))

	# list of points
	def points(self):
		(x,y,w,h) = self.raw()
		return set([vector(x,y), vector(x+w,y), vector(x+w,y+h), vector(x,y+h)])

	def get_center(self):
		(x,y,w,h) = self.raw()
		return vector(x+w/2, y+h/2)

	def intersects(self, other):
		
		# check if this rect has any of the other shape's points
		for p in other.points():
			if self.contains_p(p):
				return True

		# check if the other has any of our points
		for p in self.points():
			if other.contains_p(p):
				return True
		
		# check overlap; this is less common, for high-velocity
		if self.point_below(other.bottom_left()) and self.point_above(other.top_left()):
			return True
		if self.point_below(other.bottom_right()) and self.point_above(other.top_right()):
			return True
		if self.point_on_left(other.bottom_left()) and self.point_on_right(other.bottom_right()):
			return True
		if self.point_on_left(other.top_left()) and self.point_on_right(other.top_right()):
			return True
		

	def point_on_left(self, vect):
		return vect.x <= self.point.x

	def point_on_right(self, vect):
		return vect.x >= self.point.x + self.size.x
	
	def point_above(self, vect):
		return vect.y >= self.point.y + self.size.y

	def point_below(self, vect):
		return vect.y <= self.point.y
	
	def rect_on_left(self, other):
		(sx, sy, sw, sh) = self.raw()
		(ox, oy, ow, oh) = other.raw()
		
		return ox+ow < sx

	def rect_on_right(self, other):
		(sx, sy, sw, sh) = self.raw()
		(ox, oy, ow, oh) = other.raw()
		
		return ox > sx+sw

	def rect_above(self, other):
		(sx, sy, sw, sh) = self.raw()
		(ox, oy, ow, oh) = other.raw()
		
		return oy >= sy + sh

	def rect_below(self, other):
		(sx, sy, sw, sh) = self.raw()
		(ox, oy, ow, oh) = other.raw()
		
		return oy + oh <= sy

	



